@@ -181,6 +181,16 @@ __pmDumpStatusList(FILE *f, int numstatus, const int statuslist[])
     fprintf(f, "  status[%d]: %d\n", i, statuslist[i]);
   }
 }
+
+void
+__pmDumpNameandStatusList(FILE *f, int numnames, char *namelist[], int statuslist[])
+{
+  int i;
+  fprintf(f, "namelist & statuslist dump: numnames = %d\n", numnames);
+  for(i=0; i<numnames; i++) {
+    fprintf(f, "  name[%d]: \"%s\" (%s)\n", i, namelist[i], statuslist[i] == PMNS_LEAF_STATUS ? "leaf" : "non-leaf");
+  }
+}
 #endif
 
 /*
@@ -401,7 +411,7 @@ SendNameReq(int fd, int mode, const char *name, int pdu_type, int subtype)
 
 #ifdef PCP_DEBUG
     if (pmDebug & DBG_TRACE_PMNS)
-	fprintf(stderr, "SendNameReq: name=\"%s\"\n", name);
+	fprintf(stderr, "SendNameReq: name=\"%s\" pdu=%s subtype=%d\n", name, __pmPDUTypeStr(pdu_type), subtype);
 #endif
 
     namelen = (int)strlen(name);
