@@ -267,7 +267,17 @@ pmIDStr(pmID pmid)
     __pmID_int*	p = (__pmID_int*)&pmid;
     if (pmid == PM_ID_NULL)
 	return "PM_ID_NULL";
-    snprintf(pbuf, sizeof(pbuf), "%d.%d.%d", p->domain, p->cluster, p->item);
+    if (p->domain == DYNAMIC_PMID && p->item == 0)
+	/*
+	 * this PMID represents the base of a dynamic subtree in the PMNS
+	 * ... identified by setting the domain field to the reserved
+	 * value DYNAMIC_PMID and storing the real domain of the PMDA
+	 * that can enumerate the subtree in the cluster field, while
+	 * the item field is not used
+	 */
+	snprintf(pbuf, sizeof(pbuf), "%d.*.*", p->cluster);
+    else
+	snprintf(pbuf, sizeof(pbuf), "%d.%d.%d", p->domain, p->cluster, p->item);
     return pbuf;
 }
 
@@ -603,8 +613,8 @@ __pmPrintDesc(FILE *f, const pmDesc *desc)
 {
     char	*type;
     char	*sem;
-    int		sem_ok;
     static char	*unknownVal = "???";
+    const char	*units;
 
     if (desc->type == PM_TYPE_NOSUPPORT) {
 	fprintf(f, "    Data Type: Not Supported\n");
@@ -640,8 +650,12 @@ __pmPrintDesc(FILE *f, const pmDesc *desc)
 	    type = unknownVal;
 	    break;
     }
+    fprintf(f, "    Data Type: %s", type);
+    if (type == unknownVal)
+	fprintf(f, " (%d)", desc->type);
+
+    fprintf(f,"  InDom: %s 0x%x\n", pmInDomStr(desc->indom), desc->indom);
 
-    sem_ok = 1;
     switch (desc->sem) {
 	case PM_SEM_COUNTER:
 	    sem = "counter";
@@ -654,23 +668,19 @@ __pmPrintDesc(FILE *f, const pmDesc *desc)
 	    break;
 	default:
 	    sem = unknownVal;
-	    sem_ok = 0;
 	    break;
     }
 
-    fprintf(f, "    Data Type: %s  InDom: %s 0x%x\n", type, pmInDomStr(desc->indom), desc->indom);
-    fprintf(f, "    Semantics: %s  Units: ", sem);
-    if (sem_ok) {
-	const char	*units = pmUnitsStr(&desc->units);
-	if (*units == '\0')
-	    fprintf(f, "none\n");
-	else
-	    fprintf(f, "%s\n", units);
-    }
-    else {
-	int	*ip = (int *)&desc->units;
-	fprintf(f, "0x%x???\n", *ip);
-    }
+    fprintf(f, "    Semantics: %s", sem);
+    if (sem == unknownVal)
+	fprintf(f, " (%d)", desc->sem);
+
+    fprintf(f, "  Units: ");
+    units = pmUnitsStr(&desc->units);
+    if (*units == '\0')
+	fprintf(f, "none\n");
+    else
+	fprintf(f, "%s\n", units);
 }
 
 /*
