@@ -67,6 +67,9 @@ _setup_localhost()
 	then
 	    __opt='-n 1 localhost'
 	elif grep 'host .*packetsize .*count' $tmp >/dev/null 2>&1
+	then
+	    __opt='localhost 56 1'
+	elif grep 'host .*data_size.*npackets' $tmp >/dev/null 2>&1
 	then
 	    __opt='localhost 56 1'
 	fi
@@ -822,9 +825,16 @@ _setup()
 	perl_pmns="${PCP_PMDAS_DIR}/${iam}/pmns"
 	perl_dom="${PCP_PMDAS_DIR}/${iam}/domain.h"
 	perl -e 'use PCP::PMDA' 2>/dev/null
-	if test $? -eq 0; then
+	if test $? -eq 0
+	then
 	    eval PCP_PERL_DOMAIN=1 perl "$perl_name" > "$perl_dom"
 	    eval PCP_PERL_PMNS=1 perl "$perl_name" > "$perl_pmns"
+	elif $dso_opt || $daemon_opt
+	then
+	    :	# we have an alternative, so continue on
+	else
+	    echo 'Perl PCP::PMDA module is not installed, install it and try again'
+	    exit 1
 	fi
     fi
 
@@ -1037,7 +1047,8 @@ _install()
 		elif [ "X$pmda_type" = Xperl ]
 		then
 		    perl -e 'use PCP::PMDA' 2>/dev/null
-		    if test $? -ne 0; then
+		    if test $? -ne 0
+		    then
 			echo 'Perl PCP::PMDA module is not installed, install it and try again'
 		    else
 			break
