@@ -12,10 +12,6 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
- * 
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
  */
 
 #include "pmapi.h"
@@ -26,15 +22,15 @@
 static void 
 refresh_filesys_projects(pmInDom qindom, filesys_t *fs)
 {
-    char 		buffer[MAXPATHLEN];
+    char		buffer[MAXPATHLEN];
     project_t		*qp;
     fs_quota_stat_t	s;
     fs_disk_quota_t	d;
     size_t		idsz, devsz;
     FILE		*projects;
-    char        	*p, *idend;
-    uint32_t    	prid;
-    int         	qcmd, sts;
+    char		*p, *idend;
+    uint32_t		prid;
+    int			qcmd, sts;
 
     qcmd = QCMD(Q_XGETQSTAT, XQM_PRJQUOTA);
     if (quotactl(qcmd, fs->device, 0, (void*)&s) < 0)
@@ -75,9 +71,11 @@ refresh_filesys_projects(pmInDom qindom, filesys_t *fs)
 	qp = NULL;
 	sts = pmdaCacheLookupName(qindom, p, NULL, (void **)&qp);
 	if (sts == PMDA_CACHE_ACTIVE)	/* repeated line in /etc/projects? */
-	    continue;
+	    goto next;
 	if (sts != PMDA_CACHE_INACTIVE) {
 	    qp = (project_t *)malloc(sizeof(project_t));
+	    if (!qp)
+		goto next;
 #if PCP_DEBUG
 	    if (pmDebug & DBG_TRACE_LIBPMDA)
 		fprintf(stderr, "refresh_filesys_projects: add \"%s\"\n", p);
@@ -92,16 +90,19 @@ refresh_filesys_projects(pmInDom qindom, filesys_t *fs)
 	qp->files_used = d.d_icount;
 	qp->files_time_left = d.d_itimer;
 	pmdaCacheStore(qindom, PMDA_CACHE_ADD, p, (void *)qp);
+next:
+	free(p);
     }
     fclose(projects);
 }
 
 int
-refresh_filesys(pmInDom filesys_indom, pmInDom quota_indom)
+refresh_filesys(pmInDom filesys_indom, pmInDom quota_indom, pmInDom tmpfs_indom)
 {
     char buf[MAXPATHLEN];
     char realdevice[MAXPATHLEN];
     filesys_t *fs;
+    pmInDom indom;
     FILE *fp;
     char *path;
     char *device;
@@ -109,17 +110,15 @@ refresh_filesys(pmInDom filesys_indom, pmInDom quota_indom)
     int sts;
 
     pmdaCacheOp(quota_indom, PMDA_CACHE_INACTIVE);
+    pmdaCacheOp(tmpfs_indom, PMDA_CACHE_INACTIVE);
     pmdaCacheOp(filesys_indom, PMDA_CACHE_INACTIVE);
 
     if ((fp = fopen("/proc/mounts", "r")) == (FILE *)NULL)
 	return -errno;
 
     while (fgets(buf, sizeof(buf), fp) != NULL) {
-	if (( device = strtok(buf, " ")) == 0
-	   || strncmp(device, "/dev", 4) != 0)
+	if ((device = strtok(buf, " ")) == 0)
 	    continue;
-	if (realpath(device, realdevice) != NULL)
-	    device = realdevice;
 
 	path = strtok(NULL, " ");
 	type = strtok(NULL, " ");
@@ -130,11 +129,21 @@ refresh_filesys(pmInDom filesys_indom, pmInDom quota_indom)
 	    strncmp(type, "auto", 4) == 0)
 	    continue;
 
-	sts = pmdaCacheLookupName(filesys_indom, device, NULL, (void **)&fs);
+	indom = filesys_indom;
+	if (strcmp(type, "tmpfs") == 0) {
+	    indom = tmpfs_indom;
+	    device = path;
+	}
+	else if (strncmp(device, "/dev", 4) != 0)
+	    continue;
+	if (realpath(device, realdevice) != NULL)
+	    device = realdevice;
+
+	sts = pmdaCacheLookupName(indom, device, NULL, (void **)&fs);
 	if (sts == PMDA_CACHE_ACTIVE)	/* repeated line in /proc/mounts? */
 	    continue;
 	if (sts == PMDA_CACHE_INACTIVE) { /* re-activate an old mount */
-	    pmdaCacheStore(filesys_indom, PMDA_CACHE_ADD, device, fs);
+	    pmdaCacheStore(indom, PMDA_CACHE_ADD, device, fs);
 	    if (strcmp(path, fs->path) != 0) {	/* old device, new path */
 		free(fs->path);
 		fs->path = strdup(path);
@@ -151,7 +160,7 @@ refresh_filesys(pmInDom filesys_indom, pmInDom quota_indom)
 		    fs->path, device);
 	    }
 #endif
-	    pmdaCacheStore(filesys_indom, PMDA_CACHE_ADD, device, fs);
+	    pmdaCacheStore(indom, PMDA_CACHE_ADD, device, fs);
 	}
 	fs->flags = 0;
 
