@@ -124,7 +124,7 @@ create_client_stat(const char *client, const char *path, size_t size)
 				"not supported (current is %d)",
 				pmProgname, prefix, hdr->version, MMV_VERSION);
 		__pmMemoryUnmap(m, size);
-		return -ENOTSUP;
+		return -ENOSYS;
 	    }
 
 	    if (!hdr->g1 || hdr->g1 != hdr->g2) {
@@ -150,7 +150,7 @@ create_client_stat(const char *client, const char *path, size_t size)
 	    if (slist != NULL) {
 		slist[scnt].name = strdup(client);
 		slist[scnt].addr = m;
-		slist[scnt].pid = hdr->process;
+		slist[scnt].pid = (hdr->flags & MMV_FLAG_PROCESS)? hdr->process : 0;
 		slist[scnt].cluster = cluster;
 		slist[scnt].mcnt = 0;
 		slist[scnt].gen = hdr->g1;
@@ -384,6 +384,7 @@ map_stats(pmdaExt *pmda)
     __pmAddPMNSNode(pmns, pmid_build(pmda->e_domain, 0, 0), name);
     snprintf(name, sizeof(name), "%s.debug", prefix);
     __pmAddPMNSNode(pmns, pmid_build(pmda->e_domain, 0, 1), name);
+    mcnt = 2;
 
     if (indoms != NULL) {
 	for (i = 0; i < incnt; i++)
@@ -509,6 +510,7 @@ mmv_lookup_stat_metric_value(pmID pmid, unsigned int inst,
     mmv_disk_value_t * v;
     stats_t * s;
     int si, mi, vi;
+    int sts = PM_ERR_PMID;
 
     for (si = 0; si < scnt; si++) {
 	s = &slist[si];
@@ -520,6 +522,7 @@ mmv_lookup_stat_metric_value(pmID pmid, unsigned int inst,
 	    if (m[mi].item != id->item)
 		continue;
 
+	    sts = PM_ERR_INST;
 	    v = s->values;
 	    for (vi = 0; vi < s->vcnt; vi++) {
 		mmv_disk_metric_t * mt = (mmv_disk_metric_t *)
@@ -538,7 +541,7 @@ mmv_lookup_stat_metric_value(pmID pmid, unsigned int inst,
 	    }
 	}
     }
-    return -ENOENT;
+    return sts;
 }
 
 /*
@@ -565,9 +568,11 @@ mmv_fetchCallBack(pmdaMetric *mdesc, unsigned int inst, pmAtomValue *atom)
 	mmv_disk_metric_t * m;
 	mmv_disk_value_t * v;
 	stats_t * s;
+	int rv;
 
-	if (mmv_lookup_stat_metric_value(mdesc->m_desc.pmid, inst, &s, &m, &v) != 0)
-	    return PM_ERR_PMID;
+	rv = mmv_lookup_stat_metric_value(mdesc->m_desc.pmid, inst, &s, &m, &v);
+	if (rv < 0)
+	    return rv;
 
 	switch (m->type) {
 	    case MMV_TYPE_I32:
@@ -608,13 +613,17 @@ mmv_reload_maybe(pmdaExt *pmda)
     struct stat s;
     int need_reload = reload;
 
-    /* check if any of the generation numbers have changed (unexpected) */
+    /* check if generation numbers changed or monitored process exited */
     for (i = 0; i < scnt; i++) {
 	mmv_disk_header_t *hdr = (mmv_disk_header_t *)slist[i].addr;
 	if (hdr->g1 != slist[i].gen || hdr->g2 != slist[i].gen) {
 	    need_reload++;
 	    break;
 	}
+	if (slist[i].pid && !__pmProcessExists(slist[i].pid)) {
+	    need_reload++;
+	    break;
+	}
     }
 
     /* check if the directory has been modified */
@@ -657,23 +666,21 @@ mmv_text(int ident, int type, char **buffer, pmdaExt *ep)
     mmv_reload_maybe(ep);
     if (pmid_cluster(ident) == 0) {
 	if (pmid_item(ident) == 0) {
-	    /* mmv.reload */
-	    if (type & PM_TEXT_ONELINE)
-		*buffer = strdup("Control maps reloading");
-	    else
-		*buffer = strdup(
+	    static char reloadoneline[] = "Control maps reloading";
+	    static char reloadtext[] = 
 "Writing anything other then 0 to this metric will result in\n"
-"re-reading directory and re-mapping files.");
-		return (*buffer == NULL) ? -ENOMEM : 0;
+"re-reading directory and re-mapping files.\n";
+
+	    *buffer = (type & PM_TEXT_ONELINE) ? reloadoneline : reloadtext;
+	    return 0;
 	}
 	else if (pmid_item(ident) == 1) {
-	    /* mmv.debug */
-	    if (type & PM_TEXT_ONELINE)
-		*buffer = strdup("Debug flag");
-	    else
-		*buffer = strdup(
-"See pmdbg(1).  pmstore into this metric to change the debug value.\n");
-		return (*buffer == NULL) ? -ENOMEM : 0;
+	    static char debugoneline[] = "Debug flag";
+	    static char debugtext[] =
+"See pmdbg(1).  pmstore into this metric to change the debug value.\n";
+
+	    *buffer = (type & PM_TEXT_ONELINE) ? debugoneline : debugtext;
+	    return 0;
 	}
 	else
 	    return PM_ERR_PMID;
@@ -689,13 +696,13 @@ mmv_text(int ident, int type, char **buffer, pmdaExt *ep)
 
 	if ((type & PM_TEXT_ONELINE) && m->shorttext) {
 	    str = (mmv_disk_string_t *)((char *)s->addr + m->shorttext);
-	    *buffer = strdup(str->payload);
-	    return (*buffer == NULL) ? -ENOMEM : 0;
+	    *buffer = str->payload;
+	    return 0;
 	}
 	if ((type & PM_TEXT_HELP) && m->helptext) {
 	    str = (mmv_disk_string_t *)((char *)s->addr + m->helptext);
-	    *buffer = strdup(str->payload);
-	    return (*buffer == NULL) ? -ENOMEM : 0;
+	    *buffer = str->payload;
+	    return 0;
 	}
     }
 
@@ -758,7 +765,7 @@ mmv_store(pmResult *result, pmdaExt *ep)
 }
 
 static int
-mmv_pmid(char *name, pmID *pmid, pmdaExt *pmda)
+mmv_pmid(const char *name, pmID *pmid, pmdaExt *pmda)
 {
     mmv_reload_maybe(pmda);
     return pmdaTreePMID(pmns, name, pmid);
@@ -772,7 +779,7 @@ mmv_name(pmID pmid, char ***nameset, pmdaExt *pmda)
 }
 
 static int
-mmv_children(char *name, int traverse, char ***kids, int **sts, pmdaExt *pmda)
+mmv_children(const char *name, int traverse, char ***kids, int **sts, pmdaExt *pmda)
 {
     mmv_reload_maybe(pmda);
     return pmdaTreeChildren(pmns, name, traverse, kids, sts);
